"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var error_1 = require("../error");
var helper_1 = require("../helper");
function toTemplateLiteral(str) {
    isES5String(str);
    var QUOTA = findQuota(str);
    var LINE_BREAK = helper_1.getLineBreak(str);
    if (!LINE_BREAK) {
        return convertSingleLine(str, QUOTA);
    }
    return convertMultipleLines(str, QUOTA, LINE_BREAK);
}
exports.toTemplateLiteral = toTemplateLiteral;
function isES5String(str) {
    if (str.startsWith('`') || str.endsWith('`')) {
        throw new error_1.IncorrectInputError('not a valid concatenated string, [`] should not around the whole string');
    }
}
function findQuota(str) {
    var firstChar = str.charAt(0);
    if (firstChar === '\'' || firstChar === '"') {
        return firstChar;
    }
    return str.split('').find(function (c) { return c === '\'' || c === '"'; });
}
function convertSingleLine(line, quota) {
    var snippets = line.split(/\s*\+\s*/);
    var content = snippets.reduce(function (p, c) {
        if (!c.startsWith(quota) && !c.endsWith(quota)) {
            if (c.includes(quota) && !c.includes('\\' + quota)) {
                throw new error_1.IncorrectInputError('not a valid concatenated string, [' + quota + '] can not be in variable');
            }
            return p + '${' + c + '}';
        }
        if (c.startsWith(quota) && c.endsWith(quota)) {
            var strContent = c.slice(1, -1);
            if (strContent.includes(quota) && !strContent.includes('\\' + quota)) {
                throw new error_1.IncorrectInputError('not a valid concatenated string, [' + quota + '] can not be in plain string');
            }
            return p + strContent;
        }
        throw new error_1.IncorrectInputError('not a valid concatenated string');
    }, '');
    return '`' + content + '`';
}
function convertMultipleLines(raw, quota, lineBreak) {
    var LINE_BREAK_CHARACTERS = lineBreak === '\n' ? '\\n~~~' : '\\r\\n~~~';
    var LINE_BREAK_CHARACTERS_REG = lineBreak === '\n' ? '\\\\n' : '\\\\r\\\\n';
    var rawWithoutBreak = raw
        .split(lineBreak)
        .join(LINE_BREAK_CHARACTERS);
    var snippets = rawWithoutBreak.split(/\s*\+\s*/);
    var content = snippets.reduce(function (p, c) {
        var cWithoutbreak = c.replace(new RegExp(LINE_BREAK_CHARACTERS_REG + '~~~'), '');
        if (!cWithoutbreak) {
            return p + cWithoutbreak;
        }
        if (!cWithoutbreak.startsWith(quota) && !cWithoutbreak.endsWith(quota)) {
            if (cWithoutbreak.includes(quota) && !cWithoutbreak.includes('\\' + quota)) {
                throw new error_1.IncorrectInputError('not a valid concatenated string, [' + quota + '] can not be in variable');
            }
            return p + (c.startsWith(LINE_BREAK_CHARACTERS) ? LINE_BREAK_CHARACTERS : '') + '${' + cWithoutbreak + '}' + (c.endsWith(LINE_BREAK_CHARACTERS) ? LINE_BREAK_CHARACTERS : '');
        }
        if (cWithoutbreak.startsWith(quota) && cWithoutbreak.endsWith(quota)) {
            var strContent = cWithoutbreak.slice(1, -1);
            if (strContent.includes(quota) && !strContent.includes('\\' + quota)) {
                throw new error_1.IncorrectInputError('not a valid concatenated string, [' + quota + '] can not be in plain string');
            }
            return p + (c.startsWith(LINE_BREAK_CHARACTERS) ? LINE_BREAK_CHARACTERS : '') + strContent + (c.endsWith(LINE_BREAK_CHARACTERS) ? LINE_BREAK_CHARACTERS : '');
        }
        throw new error_1.IncorrectInputError('not a valid concatenated string');
    }, '');
    var newContent = content
        .replace(new RegExp('\\$\\{\\}', 'g'), '')
        .replace(new RegExp(LINE_BREAK_CHARACTERS_REG + '~~~', 'g'), '')
        .replace(new RegExp(LINE_BREAK_CHARACTERS_REG, 'g'), '\n');
    return '`' + newContent + '`';
}
function getVariableReplacer(quota) {
    return function (match, offset) {
        return quota + ' + ' + match.slice(2, -1) + ' + ' + quota;
    };
}
