"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var error_1 = require("../error");
var helper_1 = require("../helper");
function toConcatenatedStrings(literal, quota) {
    isTemplateLiteral(literal);
    var LINE_BREAK = helper_1.getLineBreak(literal);
    if (!LINE_BREAK) {
        return convertSingleLine(literal, quota);
    }
    return convertMultipleLines(literal, quota, LINE_BREAK);
}
exports.toConcatenatedStrings = toConcatenatedStrings;
function isTemplateLiteral(literal) {
    if (!literal || literal.length < 2 || literal.charAt(0) !== literal.charAt(literal.length - 1) || literal.charAt(0) !== '`') {
        throw new error_1.IncorrectInputError('Not a valid literal');
    }
    var content = literal.slice(1, -1);
    if (content.includes('`') && !content.includes('\\`')) {
        throw new error_1.IncorrectInputError('Not a valid literal, contains invalid [`]');
    }
}
function convertSingleLine(line, quota) {
    var content = line.slice(1, -1);
    var newContent = content
        .replace(new RegExp(quota, 'g'), getQuotaReplacer(quota))
        .replace(/\$\{.+?\}/g, getVariableReplacer(quota));
    var newLine = "" + quota + newContent + quota;
    return newLine.replace(new RegExp('^' + quota + quota + '\\s\\+\\s'), '')
        .replace(new RegExp('\\s\\+\\s' + quota + quota + '$'), '');
}
function convertMultipleLines(raw, quota, lineBreak) {
    var LINE_BREAK_CHARACTERS = lineBreak === '\n' ? '\\n' : '\\r\\n';
    var LINE_BREAK_CHARACTERS_REG = lineBreak === '\n' ? '\\\\n' : '\\\\r\\\\n';
    var content = raw
        .split(lineBreak)
        .join(LINE_BREAK_CHARACTERS)
        .slice(1, -1);
    var newContent = content.replace(new RegExp(quota, 'g'), getQuotaReplacer(quota))
        .replace(/\$\{.+?\}/g, getVariableReplacer(quota));
    var newLine = "" + quota + newContent + quota;
    return newLine.replace(new RegExp('^' + quota + quota + '\\s\\+\\s'), '')
        .replace(new RegExp('\\s\\+\\s' + quota + quota + '$'), '')
        .replace(new RegExp(LINE_BREAK_CHARACTERS_REG, 'g'), LINE_BREAK_CHARACTERS + '\' +' + lineBreak + '\'');
}
function getQuotaReplacer(quota) {
    return function (match, offset, line) {
        return (line.charAt(offset - 1) === '\\' ? '\\\\' : '\\') + quota;
    };
}
function getVariableReplacer(quota) {
    return function (match, offset) {
        return quota + ' + ' + match.slice(2, -1) + ' + ' + quota;
    };
}
